
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for testing by the author');
  }
}

use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.18

use Test::More 0.88;
use Test::EOL;

my @files = (
    'lib/Stepford.pm',
    'lib/Stepford/Error.pm',
    'lib/Stepford/FinalStep.pm',
    'lib/Stepford/LoggerWithMoniker.pm',
    'lib/Stepford/Plan.pm',
    'lib/Stepford/Planner.pm',
    'lib/Stepford/Role/Step.pm',
    'lib/Stepford/Role/Step/FileGenerator.pm',
    'lib/Stepford/Role/Step/FileGenerator/Atomic.pm',
    'lib/Stepford/Role/Step/Unserializable.pm',
    'lib/Stepford/Runner.pm',
    'lib/Stepford/Runner/State.pm',
    'lib/Stepford/Trait/StepDependency.pm',
    'lib/Stepford/Trait/StepProduction.pm',
    'lib/Stepford/Types.pm',
    'lib/Stepford/Types/Internal.pm',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/Planner.t',
    't/Runner-child-death.t',
    't/Runner-inner-steps.t',
    't/Runner-integration.t',
    't/Runner-no-unnecessary-rebuild.t',
    't/Runner-parallel-unserializable.t',
    't/Runner-parallel.t',
    't/Runner-rebuild-on-missing-files.t',
    't/Runner.t',
    't/Step-FileGenerator-Atomic-fork-bug.t',
    't/Step-FileGenerator-Atomic.t',
    't/Step.t',
    't/author-00-compile.t',
    't/author-eol.t',
    't/author-no-tabs.t',
    't/author-pod-spell.t',
    't/author-pod-syntax.t',
    't/author-test-version.t',
    't/lib/Test1/Step/CombineFiles.pm',
    't/lib/Test1/Step/CreateA1.pm',
    't/lib/Test1/Step/CreateA2.pm',
    't/lib/Test1/Step/UpdateFiles.pm',
    't/lib/Test1/StepGroup/CreateAndBackup.pm',
    't/release-cpan-changes.t',
    't/release-pod-coverage.t',
    't/release-portability.t',
    't/release-synopsis.t',
    't/release-tidyall.t'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
